; PIC SUNRISE LAMP
;Version 140509, Mikko Jrvinen
;mikko.jarvinen@windowslive.com
;Made on PIC12F509 and uses TMR0 for the PWM ramp for variable firing time of a triac
 
; An external alarm clock trigger is used.   The trigger impulce has been got from 
; a digital alarm clock's beeper signal, which is giving negative pulses for 2 minutes. These pulses were rectifiec, 
;filtered and inverted by a PNP transistor before leading to the ALARM input
; 
;The ramp counter uses the 100 Hz line clock as the counting frequency.  
;The ramp time is calculated as follows; g1 x g2 x (255-g3) x Z seconds, 
;where Z is the AC half-wave time in seconds (0,01 s for 50 Hz, 0,0083 s for 60 Hz). 
;The number of voltage steps is 255-100= 155 steps.
;
;At the beginning of the ramp a timer value 100 is written to TMR0 and TMR0 runs (255-100)*64 us before firing TRIAC.
;(TMR0 prescaler=1/64) which is a litle bit less than the 10000 us period between zero cross signals. 
;Thus the TRIAC's on-time before new Z-cross is almost zero.  The initial on-time can well be
;a a little bit longer than zero, because the incandescent lamp needs about 5% of the maximal power
;before any glow can be seen. You can adjust the initial brightness by increasing the value of g3.
;	
;At the end of the 30 min ramp TMR0 overflows immediately at the beginning of the half cycle 
;and triac is on for (almost) whole half cycle giving full brightnes.
;The triac was connected via a MOC3021 optocoupler following the instructions in the application note.
;
;Pressing  of the Master Reset button the triac can be shut down at any time. 
;The reset initializes the program and puts it to wait for a new alarm. 
;
;For use with 60 Hz AC the values of g1, g2 and g3 should be recalculated in order to get a 30 min ramp
;At 60 Hz the value of g3 is 131. 

;In order to turn the triac permanently on a harware switch has been added giving a 5 V voltage to 
;the optpocoupler's LED. The same on-off-on swich also serves to permanently shut down the lamp by disconnectin MOC3021 input
; (see the schematiics)

;******************************************************************************************************
    list        p=12F509      
    #include    <p12F509.inc>

                ; reset enabled, no code protect, no watchdog, 4MHz int clock
    __CONFIG    _MCLRE_ON & _CP_OFF & _WDT_OFF & _IntRC_OSC
 
;pins
;GPIO3 (pin 4) is configured as a reset pin and used on  hardware level for interrupting of the lamp cycle.
#define	Z_pin 	GPIO,5		;Z-input, pin 2
#define	Alarm	GPIO,4		;Alarm-input, pin 3
#define	TRIAC 	GPIO,1		;Triac-output, pin 6
#define	Ready	GPIO,2		;Ready-light, pin 5


 
;***** VARIABLE DEFINITIONS

        UDATA
; ramp variables
grad1		res	1	;ramp counter 1 
grad2		res	1	;ramp counter 2  
grad3		res	1	;ramp counter 3

;delay variables for lamp's on-time after the ramp
d1			res	1	
d2			res	1
d3			res	1
d4			res	1

;ramp constant values. 30 min= 30*38*(255-100)*100ms =180000ms 
g1	equ	.30		
g2	equ	.38		
g3	equ	.100	
				;by increasing the value of g3  you can reduce the dark delay period
				; at the beginning of the ramp.  Try value 105.  
				;Value 105 shortens the ramp (and the dark delay) by about 2 minutes.


;**************Oscillator calibration 
RCCAL   CODE    0x3FF           ; processor reset vector
        res     1               ; holds movlw with factory RC value

RESET   CODE    0x000           ; effective reset vector
        movwf   OSCCAL          ; update OSCCAL with factory cal value


;********************************the program stars from here

Alarm_loop

;pin configuration
	movlw	b'111000'		;set GPIO's 0-2 as outputs, 3-5 as inputs
	TRIS	GPIO
	clrf	GPIO			;all pins low
	
;test for alarm. My alarm pulse is positive.
;if your alarm pulse is negative, change this test to "btfsc"
	btfss	Alarm
	bsf		Ready	;set ready-light on
	goto	$-2		;if not, test again
	bcf		Ready	;if alarm was detected, set ready-light off	
MAIN
;rapm variables are loaded
	movlw	g1
	movwf	grad1
	movlw	g2
	movwf	grad2
	movlw	g3
	movwf	grad3

;set tmr0, prescaler=1/64
	movlw	b'00000101'
	option

Loop
;detect Zero Crossing peak, 100 Hz, positive peak
	btfss	Z_pin
			;if not high try again
	goto $-1
			;if set, put TRIAC down;  
	bcf		TRIAC
			;decrease grad1, if not zero, goto timer, else reload grad 1 again  and go on to grad2	
	decfsz	grad1,f	
	goto	Timer
			;reload grad1, if decreased to zero
	movlw	g1	
	movwf	grad1
			;decrease grad2, if not zero, goto timer, else goto grad2
	decfsz	grad2,f
	goto 	Timer
			;reload grad2, if decreased to zero
	movlw	g2
	movwf	grad2
			;increase grad3,
	incfsz	grad3,f	
			;move the value to Timer,
	goto	Timer
			;if grad3 overflows goto full brightnes delay
	goto 	full
	
	
Timer			
			;load TMR0 with Grad3 value
	movf	grad3,w
	movwf	TMR0
			;TRIAC is OFF, TMR0 runs up to value 255,
			;test TMR0 for value 255	
	movf	TMR0,w
	xorlw	.255
			;if values match, XORLW gives zero and Z bit of STATUS is set
			;test Z-bit
	btfss	STATUS,Z
			;if Z has not been set test TMR value again
	goto 	$-3

			;at TMR0 overflo put TRIAC high
	bsf		TRIAC
			
			;go for waiting a new Z-cross
	goto 	Loop
	
;***********when the ramp has reached the maximum value, set the lamp ON and stay there for 2 hours
; The delay can be interrupted any time by pressing the Reset button at GPIO3 (Pin4)
full
			;Put the TRIAC on
	bsf		TRIAC
	
;delay 2 h. The light is on for 2 h The code has been generated by the Microchip's "DelayCounter" program
			
			;719999998 cycles 1 MHz
	movlw	0xFF	
	movwf	d1
	movlw	0x08
	movwf	d2
	movlw	0xB0
	movwf	d3
	movlw	0x30
	movwf	d4
Delay_0
	decfsz	d1, f
	goto	$+2
	decfsz	d2, f
	goto	$+2
	decfsz	d3, f
	goto	$+2
	decfsz	d4, f
	goto	Delay_0
			;2 cycles
	goto	$+1

	bcf		TRIAC
			; after 2 hours the pogram begins to wait alarm again
		
	goto 	Alarm_loop

	END